diff -r 9624523604c5 lisp/ChangeLog
--- a/lisp/ChangeLog	Fri Feb 19 23:46:53 2010 +0000
+++ b/lisp/ChangeLog	Sun Feb 21 00:03:58 2010 +0900
@@ -0,0 +1,5 @@
+2010-02-20  Stephen J. Turnbull  <stephen@xemacs.org>
+
+	* faces.el (face-property-matching-instance):
+	Document behavior of MATCHSPEC as upward compatible from 21.4.
+
diff -r 9624523604c5 src/ChangeLog
--- a/src/ChangeLog	Fri Feb 19 23:46:53 2010 +0000
+++ b/src/ChangeLog	Sun Feb 21 00:03:58 2010 +0900
@@ -0,0 +1,10 @@
+2010-02-20  Stephen J. Turnbull  <stephen@xemacs.org>
+
+	* specifier.c (maybe_charset_to_matchspec):
+	New function for 21.4 compatibility.
+	(Fspecifier_matching_instance):
+	Use it and document new behavior for all four functions.
+	(Fspecifier_matching_instantiator):
+	(Fspecifier_matching_instance_from_inst_list):
+	(Fspecifier_matching_instantiator_from_inst_list):
+	Use it.
+

diff -r 9624523604c5 lisp/faces.el
--- a/lisp/faces.el	Fri Feb 19 23:46:53 2010 +0000
+++ b/lisp/faces.el	Sun Feb 21 00:03:58 2010 +0900
@@ -239,14 +239,18 @@
 					&optional domain default
 					no-fallback)
   "Return the instance of FACE's PROPERTY matching MATCHSPEC in DOMAIN.
-Currently the only useful value for MATCHSPEC is a charset, when used
-in conjunction with the face's font; this allows you to retrieve a
-font that can be used to display a particular charset, rather than just
-any font.
 
-Other than MATCHSPEC, this function is identical to `face-property-instance'.
-See also `specifier-matching-instance' for a fuller description of the
-matching process."
+See `face-property-instance' for details of the instancing process, and
+the semantics of FACE, PROPERTY, and DOMAIN.
+
+Currently MATCHSPEC is only useful with the 'font property.  In this case
+MATCHSPEC may be a cons \(CHARSET . STAGE) of a charset (or a symbol
+representing a charset) and a stage symbol, either 'initial or 'final.
+This allows retrieval of a font that can be used to display a particular
+charset, rather than just any font.  For backward compatibility with 21.4,
+MATCHSPEC may also be a charset or charset symbol.
+
+See `specifier-matching-instance' for details of MATCHSPEC processing."
 
   (setq face (get-face face))
   (let ((value (get face property)))
diff -r 9624523604c5 src/specifier.c
--- a/src/specifier.c	Fri Feb 19 23:46:53 2010 +0000
+++ b/src/specifier.c	Sun Feb 21 00:03:58 2010 +0900
@@ -3190,6 +3190,13 @@
 				 no_fallback, 1);
 }
 
+/* Allows code written for 21.4 to continue to work. */
+static Lisp_Object
+maybe_charset_to_matchspec (Lisp_Object spec)
+{
+  return Ffind_charset (spec) ? Fcons (spec, Qinitial) : spec;
+}
+
 DEFUN ("specifier-matching-instance", Fspecifier_matching_instance, 2, 5, 0, /*
 Return an instance for SPECIFIER in DOMAIN that matches MATCHSPEC.
 If no instance can be generated for this domain, return DEFAULT.
@@ -3207,7 +3214,7 @@
    display table is not there. (Chartable specifiers are not yet
    implemented.)
 
--- For font specifiers, MATCHSPEC should be a cons (CHARSET . STAGE).
+-- For font specifiers, MATCHSPEC should be a cons \(CHARSET . STAGE).
    The defined stages are currently `initial' and `final'.  On X11, 'initial
    is used when the font matching process is looking for fonts that match
    the desired registries of the charset--see the `charset-registries'
@@ -3218,6 +3225,9 @@
    registry and encoding used will be `iso10646-1', and the characters will
    be converted to display using that registry.
 
+   For backward compatibility with Lisp code written for 21.4, a bare
+   charset CHARSET is accepted and interpreted as \(CHARSET . initial).
+
    See `define-specifier-tag' for details on how to create a tag that
    specifies a given character set and stage combination.  You can supply
    such a tag to `set-face-font' in order to set a face's font for that
@@ -3225,6 +3235,7 @@
 */
        (specifier, matchspec, domain, default_, no_fallback))
 {
+  matchspec = maybe_charset_to_matchspec (matchspec);
   return specifier_matching_foo (specifier, matchspec, domain, default_,
 				 no_fallback, 0);
 }
@@ -3240,6 +3251,7 @@
 */
        (specifier, matchspec, domain, default_, no_fallback))
 {
+  matchspec = maybe_charset_to_matchspec (matchspec);
   return specifier_matching_foo (specifier, matchspec, domain, default_,
 				 no_fallback, 1);
 }
@@ -3320,6 +3332,7 @@
 */
        (specifier, matchspec, domain, inst_list, default_))
 {
+  matchspec = maybe_charset_to_matchspec (matchspec);
   return specifier_matching_foo_from_inst_list (specifier, matchspec,
 						domain, inst_list, default_,
 						0);
@@ -3335,6 +3348,7 @@
 */
        (specifier, matchspec, domain, inst_list, default_))
 {
+  matchspec = maybe_charset_to_matchspec (matchspec);
   return specifier_matching_foo_from_inst_list (specifier, matchspec,
 						domain, inst_list, default_,
 						1);

